#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery
from selenium import webdriver

from .. import serial, utils

BASE_URL = 'http://www.aastory.xyz/archive.php?id={}'


class AastoryPage(serial.Page):

    def get_content(self):
        driver = webdriver.PhantomJS()
        driver.get(self.url)
        content = driver.find_element_by_id('chapter_content').text
        driver.close()
        return content


class Aastory(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), None,
                         chap_sel='li',
                         chap_type=serial.ChapterType.path,
                         tid=tid)
        self.page = AastoryPage

    def get_title_and_author(self):
        name = self.doc('h1').text()
        st = self.doc('.index_info').text()
        author = re.match(r'作者：(.*)', st).group(1)
        return name, author

    def get_intro(self):
        intro = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'Description'
        ).attr('content')
        return intro



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import textwrap

import prettytable
from readchar import readchar
from termcolor import colored, cprint

from . import sources
from .config import save_novel_list, GOAGENT
from .db import new_session
from .factory import add_novel
from .models import Serial, Website, Article, General
from .utils import get_filename


class NovelCmdline(object):

    def __init__(self, source=None, tids=None, http_proxy=None, verbose=None):
        self.source = source
        self.tids = tids or []
        self.http_proxy = http_proxy
        self.verbose = verbose or 0

    def __enter__(self):
        self.session = new_session()
        return self

    # noinspection PyUnusedLocal
    def __exit__(self, *args):
        self.save()
        self.session.flush()
        self.session.close()

    def save(self):
        nl = {w.name: [s.id for s in w.novels]
              for w in self.session.query(Website).all()}
        save_novel_list(nl)

    def list(self, source=None):
        source = source or self.source

        if source:
            if source in sources.SERIAL_TYPE:
                return self.list_serial(source)
            elif source in sources.ARTICLE_TYPE:
                return self.list_article(source)
            else:
                print('The specific source `{}` does not exists!'.format(source))
        else:
            novel_list = self.session.query(General).all()

            pt = prettytable.PrettyTable()
            pt.field_names = ['id', 'title', 'source']
            pt.valign = 'm'
            for novel in novel_list:
                pt.add_row((novel.id, colored(novel.title, 'green'), novel.source))

            print(pt.get_string())
            return pt

    def list_article(self, source=None):
        source = source or self.source

        if source:
            novel_list = self.session.query(Article).filter_by(
                source=source
            ).all()
        else:
            novel_list = self.session.query(Article).all()

        pt = prettytable.PrettyTable()
        pt.field_names = ['id', 'title', 'source']
        pt.valign = 'm'
        for novel in novel_list:
            pt.add_row((novel.id, colored(novel.title, 'green'), novel.source))

        if self.verbose > 0:
            length_list = [len(novel.text) for novel in novel_list]
            pt.add_column('length', length_list)

        print(pt.get_string())
        return pt

    def list_serial(self, source=None):
        source = source or self.source

        if source:
            novel_list = self.session.query(Serial).filter_by(
                source=source
            ).all()
        else:
            novel_list = self.session.query(Serial).all()

        pt = prettytable.PrettyTable()
        pt.field_names = ['id', 'title', 'author', 'source']
        pt.valign = 'm'
        for novel in novel_list:
            pt.add_row((novel.id, colored(novel.title, 'green'), novel.author, novel.source))
        if self.verbose > 0:
            pt.hrules = prettytable.ALL
            intro_list = [textwrap.fill(novel.intro, width=40)
                          for novel in novel_list]
            pt.add_column('intro', intro_list, align='l')
        if self.verbose > 1:
            pt.add_column(
                'finish', [novel.finish for novel in novel_list], valign='m')
        if self.verbose > 2:
            pt.add_column(
                'chapters', [len(novel.chapters) for novel in novel_list], valign='m')

        print(pt.get_string())
        return pt

    def delete_serial(self, source, tid):
        novel = self.session.query(Serial).filter_by(
            source=source, id=tid
        ).one()
        for ch in novel.chapters:
            self.session.delete(ch)
        self.session.delete(novel)

    def delete_article(self, source, tid):
        article = self.session.query(Article).filter_by(
            source=source, id=tid
        ).one()
        self.session.delete(article)

    def delete(self, source=None, tids=None):
        source = source or self.source
        tids = tids or self.tids

        if source:
            if tids:
                novel_list = self.session.query(General).filter(
                    General.source == source, General.id.in_(tids)
                ).all()
            else:
                novel_list = self.session.query(General).filter_by(
                    source=source
                ).all()
        else:
            novel_list = self.session.query(General).all()

        deleted = {}

        for novel in novel_list:
            deleted[novel.source] = deleted.get(novel.source, []) + [novel.id]
            if novel.source in sources.SERIAL_TYPE:
                self.delete_serial(novel.source, novel.id)
            elif novel.source in sources.ARTICLE_TYPE:
                self.delete_article(novel.source, novel.id)
            else:
                print('Something strange may happens here.')

        for s, t in deleted.items():
            print('{}: {}'.format(s, t))

        return deleted

    def update(self, source=None, tids=None):
        source = source or self.source
        tids = tids or self.tids

        if source:
            if tids:
                for tid in tids:
                    add_novel(source, tid,
                              self.http_proxy, self.session)
                return
            else:
                novel_list = self.session.query(Serial).filter_by(
                    source=source, finish=False
                ).all()
        else:
            novel_list = self.session.query(Serial).filter_by(
                finish=False
            ).all()

        for novel in novel_list:
            add_novel(novel.source, novel.id,
                      self.http_proxy, self.session)

    def dump_novel(self, source, tid):
        if source in sources.SERIAL_TYPE:
            self.dump_serial(source, tid)
        elif source in sources.ARTICLE_TYPE:
            self.dump_article(source, tid)
        else:
            print('The specific source `{}` does not exists!'.format(source))

    def dump_serial(self, source, tid):
        novel = self.session.query(Serial).filter_by(
            source=source, id=tid
        ).one()
        filename = get_filename(novel.title, novel.author)
        print(filename)

        with open(filename, 'w') as fp:
            fp.write(novel.title)
            fp.write('\n\n')
            fp.write(novel.author)
            fp.write('\n\n\n')
            fp.write(novel.intro)
            for ch in novel.chapters:
                fp.write('\n\n\n\n')
                fp.write(ch.title)
                fp.write('\n\n\n')
                fp.write(ch.text)
                fp.write('\n')

    def dump_article(self, source, tid):
        novel = self.session.query(Article).filter_by(
            source=source, id=tid
        ).one()
        filename = get_filename(novel.title)
        print(filename)

        with open(filename, 'w') as fp:
            fp.write(novel.title)
            fp.write('\n\n\n\n')
            fp.write(novel.text)
            fp.write('\n')

    def dump(self, source=None, tids=None):
        source = source or self.source
        tids = tids or self.tids

        if source:
            if tids:
                for tid in tids:
                    self.dump_novel(source, tid)
            else:
                site = self.session.query(Website).filter_by(name=source).one()
                for novel in site.novels:
                    self.dump_novel(source, novel.id)
        else:
            novel_list = self.session.query(General).all()
            for novel in novel_list:
                self.dump_novel(novel.source, novel.id)

    def mark_finish(self, source=None, tids=None):
        source = source or self.source
        tids = tids or self.tids

        if source:
            if tids:
                novel_list = self.session.query(Serial).filter(
                    Serial.source == source, Serial.id.in_(tids)
                ).all()
            else:
                novel_list = self.session.query(Serial).filter_by(
                    source=source
                ).all()
        else:
            novel_list = self.session.query(Serial).all()

        for novel in novel_list:
            novel.finish = True

    def try_mark_finish(self, source=None, tids=None):
        source = source or self.source
        tids = tids or self.tids
        mark_all = False

        if source:
            if tids:
                novel_list = self.session.query(Serial).filter(
                    Serial.source == source, Serial.id.in_(tids)
                ).all()
            else:
                novel_list = self.session.query(Serial).filter_by(
                    source=source, finish=False
                ).all()
        else:
            novel_list = self.session.query(Serial).filter_by(
                finish=False
            ).all()

        def try_mark_novel(n):
            nonlocal mark_all
            print(n.id, colored(n.title, 'green'), n.author, n.source)
            if mark_all:
                n.finish = True
            else:
                cprint('Mark ths novel as finished? [!/y/N/q/u/?]', 'cyan')
                yes = readchar().lower()
                if yes == "!":
                    mark_all = True
                    n.finish = True
                elif yes == 'y':
                    n.finish = True
                elif yes == 'q':
                    return -1
                elif yes == 'u':
                    n.finish = False
                elif yes == 'n' or yes == '\r':
                    return
                else:
                    cprint('! - mark this and the following novels as finished', 'red')
                    cprint('y - mark this novel as finished', 'red')
                    cprint('n - do not change finish status', 'red')
                    cprint('q - quit; do not change finish status of this and any other novels', 'red')
                    cprint('u - mark this novel as unfinished', 'red')
                    cprint('? - print help', 'red')
                    return try_mark_novel(n)

        for novel in novel_list:
            res = try_mark_novel(novel)
            print()
            if res == -1:
                break

    def refresh(self, source=None, tids=None):
        source = source or self.source
        tids = tids or self.tids

        deleted = self.delete(source, tids)
        for s, t in deleted.items():
            self.update(s, t)


def get_schema(s):
    if s in sources.SERIAL_TYPE:
        return Serial
    elif s in sources.ARTICLE_TYPE:
        return Article
    else:
        raise NotImplementedError(s)


def get_class(s):
    return getattr(sources, s.capitalize())


def get_proxies(s):
    if s in sources.CERNET_USE_PROXIES:
        return GOAGENT
    else:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from contextlib import contextmanager

from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.pool import SingletonThreadPool

from .config import CACHE_DB
from .models import Base


@contextmanager
def create_session(db=CACHE_DB, pool_size=100):
    session = new_session(db, pool_size)
    try:
        yield session
        session.flush()
    except:
        session.rollback()
        raise
    else:
        session.close()


def new_session(db=CACHE_DB, pool_size=100):
    engine = create_engine(
        'sqlite:///' + db,
        poolclass=SingletonThreadPool,
        pool_size=pool_size
    )
    db_session = scoped_session(sessionmaker(
        bind=engine,
        autoflush=True,
        autocommit=True
    ))
    session = db_session()
    Base.metadata.create_all(engine)
    return session



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from urllib.parse import urljoin

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.lwxs520.com/books/{}/{}/'
INTRO_URL = 'http://www.lwxs520.com/book/{}/'


class Lwxs520Tool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.extend((
            re.compile(r'www\.lwxs520\.com 首发哦亲', re.I),
            re.compile(r'乐文小说网'),
            re.compile(r'乐文 小说'),
            re.compile(r'www\.lwxs520\.com'),
        ))


class Lwxs520(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         utils.base_to_url(INTRO_URL, tid), '.intro',
                         chap_sel='.bookinfo_td td',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB
        self.tool = Lwxs520Tool

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'author'
        ).attr('content')
        return re.match(r'(.*)版权属于作者(.*)', st).groups()

    @property
    def chapter_list(self):
        clist = self.doc(self.chap_sel).filter(
            lambda i, e: PyQuery(e)('a').attr('href')
        ).map(
            lambda i, e: (i,
                          urljoin(self.url, PyQuery(e)('a').attr('href')),
                          self.simple_refine(PyQuery(e).text()))
        )
        return clist

    @staticmethod
    def simple_refine(text):
        text = re.sub(re.compile(r'www\.lwxs520\.com', re.I), '', text)
        text = re.sub(re.compile(r'乐文小说网'), '', text)
        return text



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import traceback
from functools import wraps
from time import sleep


def retry(ExceptionToTrack, tries=5, delay=5):
    def dec_retry(func):
        @wraps(func)
        def func_retry(*args, **kwargs):
            m_delay = delay
            for _ in range(tries):
                try:
                    return func(*args, **kwargs)
                except ExceptionToTrack:
                    traceback.print_exc()
                    print('Wait {:d} seconds to retry ...'.format(m_delay))
                    sleep(m_delay)
                    m_delay *= 2
            return func(*args, **kwargs)

        return func_retry

    return dec_retry



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# from sqlalchemy import *
from sqlalchemy import Column, Integer, String, Text, ForeignKeyConstraint, Boolean
from sqlalchemy.ext.declarative import as_declarative, declared_attr
from sqlalchemy.orm import relationship


@as_declarative()
class Base(object):

    # noinspection PyMethodParameters
    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower()

    def __repr__(self):
        return '<{}({})>'.format(type(self).__name__, self.column_items_ignore_text)

    @property
    def columns(self):
        return self.__table__.columns.keys()

    @property
    def column_items(self):
        return {c: getattr(self, c) for c in self.columns}

    @property
    def column_items_ignore_text(self):
        return {k.name: self._short_item(k)
                for k in self.__table__.columns.values()}

    def _short_item(self, k):
        if isinstance(k.type, Text):
            return '{} characters'.format(len(getattr(self, k.name)))
        else:
            return getattr(self, k.name)

    def to_json(self):
        return self.column_items


class Website(Base):
    name = Column(String, primary_key=True)
    url = Column(String)

    novels = relationship('General', backref='website',
                          order_by='General.id')


class General(Base):
    id = Column(String, primary_key=True)
    title = Column(String)
    source = Column(String, primary_key=True)

    ForeignKeyConstraint(
        [source], [Website.name]
    )


# serial.SerialNovel
class Serial(General):
    id = Column(String, primary_key=True)
    title = Column(String)
    author = Column(String)
    intro = Column(Text)
    source = Column(String, primary_key=True)
    finish = Column(Boolean, default=False)

    chapters = relationship('Chapter', backref='novel',
                            order_by='Chapter.id')

    ForeignKeyConstraint(
        [id, source, title],
        [General.id, General.source, General.title]
    )


# serial.Page
class Chapter(Base):
    id = Column(Integer, primary_key=True)
    url = Column(String)
    title = Column(String)
    text = Column(Text)

    novel_id = Column(Integer, primary_key=True)
    novel_source = Column(String, primary_key=True)

    ForeignKeyConstraint(
        [novel_id, novel_source],
        [Serial.id, Serial.source]
    )


# single.SingleNovel
class Article(General):
    id = Column(String, primary_key=True)
    title = Column(String)
    text = Column(Text)
    source = Column(String, primary_key=True)

    ForeignKeyConstraint(
        [id, title, source],
        [General.id, General.title, General.source]
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.33yq.com/read/{}/{}/'


class Yq33Tool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.replace_extras.append(
            (re.compile(r'123言情'), '晋江')
        )


class Yq33(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#TXT',
                         None, '.introtxt',
                         tid=tid)
        self.encoding = config.GB
        self.tool = Yq33Tool

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords').attr('content')
        return re.match(r'(.*?),(.*?),', st).groups()

    @property
    def chapter_list(self):
        clist = self.doc('dd').map(
            lambda i, e: (utils.fix_order(i),
                          PyQuery(e)('a').attr('href'),
                          PyQuery(e).text())
        ).filter(
            lambda i, e: e[1]
        )
        clist.sort()
        return clist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

SERIAL_TYPE = (
    'aastory',
    'bgif2',
    'biquge',
    'danmei123',
    'dzxsw',
    'feizw',
    'haxtxt',
    'icnsp',
    'klxsw',
    'lwxs',
    'lwxs520',
    'lwxsw',
    'piaotian',
    'piaotiancc',
    'ranwen',
    'shu69',
    'shushu8',
    'sto',
    'ttshuba',
    'ttzw',
    'ttzw5',
    'uks5',
    'wdxs',
    'xs365',
    'yfzww',
    'yq33',
    'zhaishu8',
)

ARTICLE_TYPE = (
    'cool18',
    'doubangroup',
    'park6',
    'sdragon',
    'sis',
)

# Configuration for free ip in CERNET
CERNET_USE_PROXIES = (
    'cool18',
    'danmei123',
    'dzxsw',
    'haxtxt',
    'klxsw',
    'lwxs520',
    'lwxs',
    'lwxsw',
    'park6',
    'piaotian',
    'piaotiancc',
    'ranwen',
    'sdragon',
    'shu69',
    'shushu8',
    'sis',
    'sto',
    'ttshuba',
    'ttzw',
    'uks5',
    'wdxs',
    'xs365',
    'yq33',
    'zhaishu8',
)

DEFAULT_NOT_OVERWRITE = (
    'cool18',
    'park6',
)

AUTO_MARK_FINISH = (
    'sto',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Some help functions
"""

import collections
import itertools
import os
import re
import string
from urllib.parse import urlparse, urlunparse

from .config import save_novel_list, load_novel_list
from .db import create_session
from .factory import add_novel
from .models import Website


class Tool(object):
    """
    A class to remove needless tags and strings
    """

    def __init__(self):
        self._remove_a = re.compile(r'<a.*?>.*?</a>', re.I)
        self._remove_div = re.compile(r'<div.*?>.*?</div>',
                                      re.I | re.S)
        self._remove_span = re.compile(r'<span.*?>.*?</span>',
                                       re.I | re.S)
        self._remove_font = re.compile(r'<font.*?>.*?</font>',
                                       re.I | re.S)
        self._remove_script = re.compile(r'<script.*?>.*?</script>',
                                         re.I | re.S)
        self._replace_br = re.compile(r'<br\s*/\s*>|</\s*br>', re.I)
        self._replace_p = re.compile(r'</?p>', re.I)
        self._replace_h = re.compile(r'</h\d*?>')
        self._replace_xa0 = re.compile(r'\xa0')
        self._replace_u3000 = re.compile(r'\u3000')
        self._remove_ufeff = re.compile(r'\ufeff')
        self._remove_r = re.compile(r'&#13;|\r')
        self._replace_gt = re.compile(r'&gt;')
        self._replace_lt = re.compile(r'&lt;')
        self.replace_extras = []
        self.remove_extras = []
        self._remove_ot = re.compile(r'<.*?>')

    def replace(self, text):
        """
        Replace and remove needless strings

        Some default options are pre-defined.
        You can add custom options to replace_extras or remove_extras.
        """
        text = re.sub(self._remove_a, '', text)
        text = re.sub(self._remove_div, '', text)
        text = re.sub(self._remove_span, '', text)
        text = re.sub(self._remove_font, '', text)
        text = re.sub(self._remove_script, '', text)
        text = re.sub(self._replace_br, '\n', text)
        text = re.sub(self._replace_p, '\n', text)
        text = re.sub(self._replace_h, '\n\n', text)
        text = re.sub(self._replace_xa0, ' ', text)
        text = re.sub(self._replace_u3000, '  ', text)
        text = re.sub(self._remove_ufeff, '', text)
        text = re.sub(self._remove_r, '', text)
        text = re.sub(self._replace_gt, '>', text)
        text = re.sub(self._replace_lt, '<', text)
        for s, d in self.replace_extras:
            text = re.sub(s, d, text)
        for pat in self.remove_extras:
            text = re.sub(pat, '', text)
        text = re.sub(self._remove_ot, '', text)
        return text

    def refine(self, text):
        """
        Get a better printed text

        Replace and remove needless strings, and then
        remove too many continuous newlines.
        """
        text = self.replace(text)

        text = re.sub(r'\n\s+\n', '\n\n', text)
        text = re.sub(r'[ \t]+\n', '\n', text)

        return text.strip()


def fix_order(i):
    """
    Fix the order of list item.

    Sometimes we get a list of order
    [0<2>, 1<1>, 2<0>, 3<5>, 4<4>, 5<3>, ...],
    and what we need is [2<0>, 1<1>, 0<2>, 5<3>, 4<4>, 3<5>, ...].
    """
    if i % 3 == 0:
        return i + 2
    elif i % 3 == 2:
        return i - 2
    else:
        return i


def count(iterable):
    """
    Count the number of items that `iterable` yields

    Equivalent to the expression

    ::
      len(iterable),

    ... but it also works for iterables that do not support ``len()``.

    ::

      >>> import cardinality
      >>> cardinality.count([1, 2, 3])
      3
      >>> cardinality.count(i for i in range(500))
      500
      >>> def gen():
      ...     yield 'hello'
      ...     yield 'world'
      >>> cardinality.count(gen())
      2

    Get from https://github.com/wbolster/cardinality
    """
    if hasattr(iterable, '__len__'):
        return len(iterable)

    d = collections.deque(enumerate(iterable, 1), maxlen=1)
    return d[0][0] if d else 0


def get_field_count(format_string):
    fmt = string.Formatter()
    return count(t for t in fmt.parse(format_string) if t[1] is not None)


def base_to_url(base_url, tid):
    """
    Get the url from template

    The base_url must have two replacement fields.
    The second field is just filled with the tid,
    while the first field with the tid which has been
    stripped the last three number.
    """
    field_count = get_field_count(base_url)
    if field_count == 1:
        return base_url.format(tid)
    elif field_count == 2:
        return base_url.format(int(tid) // 1000, tid)
    else:
        raise Exception(
            'Function base_to_url with {} replacement fields is not defined!'.format(field_count))


def get_base_url(url):
    """
    Transform a full url into its base url

    For example:

    ::
      >>> get_base_url('http://example.com/text/file?var=f')
      'http://example.com'
    """
    result = urlparse(url)
    base_url = urlunparse((result.scheme, result.netloc, '', '', '', ''))
    return base_url


def get_filename(title, author=None, overwrite=True):
    if author:
        base = '《{}》{}'.format(title, author)
    else:
        base = title

    filename = '{}.txt'.format(base)
    filename = filename.replace('/', '_')
    if not overwrite:
        if os.path.exists(filename):
            for i in itertools.count(1):
                filename = '{}({:d}).txt'.format(base, i)
                if not os.path.exists(filename):
                    break
    return filename


def sync_db_to_list():
    with create_session() as session:
        nl = {}
        for w in session.query(Website).all():
            nl[w.name] = [s.id for s in w.novels]
    save_novel_list(nl)


def sync_list_to_db():
    nl = load_novel_list()
    for s, tids in nl.items():
        for tid in tids:
            add_novel(s, tid)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .. import serial, utils, config

BASE_URL = 'http://www.haxtxt.com/files/article/html/{}/{}/'


class Haxtxt(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#BookText',
                         None, '.intro',
                         chap_sel='.chapterlist dd',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        title = self.doc('.btitle h1').text().strip('《》')
        author = self.doc('.btitle em a').text()
        return title, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils

BASE_URL = 'http://www.yfzww.com/Book/{}'


class YfzwwTool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.append(
            re.compile(r'【一凡中文网.*', re.S)
        )


class Yfzww(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         chap_sel='#chapters li',
                         chap_type=serial.ChapterType.path,
                         tid=tid)
        self.tool = YfzwwTool

    def get_title_and_author(self):
        name = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:book_name'
        ).attr('content')

        author = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:author'
        ).attr('content')

        return name, author

    def get_intro(self):
        intro = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:description'
        ).attr('content')
        return intro



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.lwxs.com/shu/{}/{}/'
INTRO_URL = 'http://www.lwxs.com/info-{}.html'


class Lwxs(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '.zhangjieTXT',
                         utils.base_to_url(INTRO_URL, tid), '.intro',
                         chap_sel='dd',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords'
        ).attr('content')
        return re.match(r'(.*)最新章节\((.*?)\),.*', st).groups()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from urllib.parse import urljoin

from pyquery import PyQuery
from selenium import webdriver

from .. import serial, utils, config

BASE_URL = 'http://www.piaotian.net/html/{}/{}/'
INTRO_URL = 'http://www.piaotian.net/bookinfo/{}/{}.html'


class PiaotianPage(serial.Page):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.encoding = config.GB

    def get_content(self):
        content = self.doc.html()
        pat = re.compile(r'.*<!-- 标题上AD结束 -->(.*)<!-- 翻页上AD开始 -->.*',
                         re.S)
        try:
            content = re.match(pat, content).group(1)
        except AttributeError:
            if 'http' in self.proxies:
                service_args = [
                    '--proxy=' + self.proxies['http'],
                    '--proxy-type=http'
                ]
            else:
                service_args = None
            driver = webdriver.PhantomJS(service_args=service_args)
            driver.get(self.url)
            driver.execute_script('''
                var element = document.querySelector(".toplink");
                if (element)
                    element.parentNode.removeChild(element);
            ''')
            content = driver.find_element_by_id('content').text
            driver.close()
            return content
        content = self.refine(content)
        return content


class PiaotianIntroPageTool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.append(
            re.compile(r'.*</span>')
        )


class PiaotianIntroPage(serial.IntroPage):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.tool = PiaotianIntroPageTool
        self.encoding = config.GB

    def get_content(self):
        intro = self.doc('div').filter(
            lambda i, e: 'float:left' in (PyQuery(e).attr('style') or '')
        ).html()
        intro = self.refine(intro)
        return intro


class PiaotianTool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.extend((
            re.compile(r'飘天文学'),
            re.compile(r'www\.piaotian\.com', re.I),
            re.compile(r'&lt;tr&gt;&lt;td&gt;'),
            re.compile(r'&lt;div id="content"&gt;\xa0\xa0\xa0\xa0'),
            re.compile(r'&amp;nbsp'),
            re.compile(r'手机用户请访问http://m\.piaotian\.net'),
        ))


class Piaotian(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), None,
                         utils.base_to_url(INTRO_URL, tid), None,
                         tid=tid)
        self.encoding = config.GB
        self.tool = PiaotianTool
        self.page = PiaotianPage
        self.intro_page = PiaotianIntroPage

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords'
        ).attr('content')
        name = re.match(r'(.*?),.*', st).group(1)
        author = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'author'
        ).attr('content')
        return name, author

    @property
    def chapter_list(self):
        clist = self.doc('li').filter(
            lambda i, e: (PyQuery(e)('a').attr('href') and
                          re.match(r'\d+\.html', PyQuery(e)('a').attr('href')))
        ).map(
            lambda i, e: (i,
                          urljoin(self.url, PyQuery(e)('a').attr('href')),
                          PyQuery(e).text())
        )
        return clist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils

BASE_URL = 'http://www.sto.cc/{}-1/'
PAGE_URL = 'http://www.sto.cc/{}-{}/'


class StoTool(utils.Tool):

    def __init__(self):
        super().__init__()

        word_list = (
            's思s兔s網s文s檔s下s載s與s在s線s閱s讀s',
            's本s作s品s由s思s兔s網s提s供s下s載s與s在s線s閱s讀s',
            's本s作s品s由s思s兔s在s線s閱s讀s網s友s整s理s上s傳s',
            's思s兔s在s線s閱s讀s',
            's思s兔s文s檔s共s享s與s在s線s閱s讀s',
        )
        symbol_list = (
            '\^_\^', ':-\)', '\^o\^', '-_-!',
            '││', '//', '\$\$',
        )
        symbols = '|'.join(symbol_list).join(('(.|', ')'))
        pats = (symbols.join(w.split('s')) for w in word_list)

        symbol_extras = ('',)

        self.remove_extras.extend(
            (re.compile(pat) for pat in pats)
        )
        self.remove_extras.extend(
            (re.compile(pat) for pat in symbol_extras)
        )


class Sto(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#BookContent',
                         tid=tid)
        self.tool = StoTool

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords'
        ).attr('content')
        return re.match(r'(.*?),(.*?),.*', st).groups()

    @property
    def chapter_list(self):
        st = re.search(
            r'ANP_goToPage\("Page_select",(\d+),(\d+),1\);', self.doc.html())
        if st.group(1) == self.tid:
            page_num = int(st.group(2))
        else:
            raise Exception('Something strange may happened.')
        return [(i + 1, PAGE_URL.format(self.tid, i + 1), '第{:d}頁'.format(i + 1))
                for i in range(page_num)]

    def get_intro(self):
        intro = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'description'
        ).attr('content')
        return intro



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://tw.piaotian.cc/read/{}/index.html'
INTRO_URL = 'http://tw.piaotian.cc/book/{}.html'


class Piaotiancc(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '.novel_content',
                         utils.base_to_url(INTRO_URL, tid), '#description1',
                         chap_sel='.novel li',
                         chap_type=serial.ChapterType.path,
                         tid=tid)
        self.encoding = config.BIG

    def get_title_and_author(self):
        title = self.doc('h1').text()
        author = self.doc('.novel_info')('a').filter(
            lambda i, e: PyQuery(e).attr('target') == '_blank'
        ).text()
        return title, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from urllib.parse import urljoin

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.69shu.com/{}/'
INTRO_URL = 'http://www.69shu.com/modules/article/jianjie.php?id={}'


class Shu69(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '.yd_text2',
                         utils.base_to_url(INTRO_URL, tid), '.jianjie',
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords'
        ).attr('content')
        name = re.match(r'(.*?),.*', st).group(1)
        author = self.doc('.mu_beizhu').eq(0)('a').eq(1).text()
        return name, author

    @property
    def chapter_list(self):
        clist = self.doc('.mulu_list').eq(1)('li').filter(
            lambda i, e: PyQuery(e)('a').attr('href')
        ).map(
            lambda i, e: (i,
                          urljoin(utils.get_base_url(self.url),
                                  PyQuery(e)('a').attr('href')),
                          PyQuery(e).text())
        )
        return clist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.wodexiaoshuo.com/{}/chapter.html'
INTRO_URL = 'http://www.wodexiaoshuo.com/{}/'


class WdxsTool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.extend(
            (re.compile(pat, re.I) for pat in
             (r'www\.(wodexiaoshuo|01bz)\.(com|net|wang)',
              r'wodexiaoshuo\.com',
              r'www\.'))
        )
        self.remove_extras.extend(
            (re.compile(pat) for pat in
             (r'\t',
              r'&amp;nbsp;'))
        )


class Wdxs(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '.box_box',
                         utils.base_to_url(INTRO_URL, tid), '.j_box .words',
                         chap_sel='.box_box li',
                         chap_type=serial.ChapterType.path,
                         tid=tid)
        self.encoding = config.GB
        self.tool = WdxsTool

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords'
        ).attr('content')
        name = re.match(r'(.*?),.*', st).group(1)
        author = self.doc('a').filter(
            lambda i, e: re.match(r'^/author/\?\d+\.html$',
                                  PyQuery(e)('a').attr('href') or '')
        ).attr('title')
        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.danmei123.com/{}/'


class Danmei123(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '.box_box',
                         None, '.j_box .words p',
                         chap_sel='.list_box li',
                         chap_type=serial.ChapterType.path,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'description'
        ).attr('content')
        pat = re.compile(r'(.*)的新书(.*)最新章节.*')
        author, name = re.match(pat, st).groups()

        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from contextlib import contextmanager
from importlib import import_module

from . import sources
from .config import GOAGENT


@contextmanager
def use_novel(source, tid, http_proxy=None, session=None):
    module = import_module('.sources.' + source, 'novel')
    novel_class = getattr(module, source.capitalize())
    nov = novel_class(tid)
    nov.use_session(session)

    if http_proxy:
        if http_proxy != '---':
            nov.proxies = {'http': http_proxy}
    elif source in sources.CERNET_USE_PROXIES:
        nov.proxies = GOAGENT

    if source in sources.AUTO_MARK_FINISH:
        nov.finish = True

    if source in sources.DEFAULT_NOT_OVERWRITE:
        nov.overwrite = False

    yield nov

    nov.close()


def add_novel(source, tid, http_proxy=None, session=None):
    with use_novel(source, tid, http_proxy, session) as nov:
        nov.run()


def dump_novel(source, tid, http_proxy=None, session=None):
    with use_novel(source, tid, http_proxy, session) as nov:
        nov.run()
        nov.dump()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.ttzw5.com/book/{}/{}/'


class Ttzw5(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#contents',
                         chap_sel='li.zp_li',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords'
        ).attr('content')
        name = re.match(r'(.*?),.*', st).group(1)

        st = self.doc('h3').text()
        author = re.match(r'作者：(.*?)/.*', st).group(1)

        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.ranwen.org/files/article/{}/{}/'


class Ranwen(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         chap_sel='dd',
                         chap_type=serial.ChapterType.whole,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        name = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:book_name'
        ).attr('content')

        author = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:author'
        ).attr('content')

        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import requests

from .. import base, utils, models, db

BASE_URL = 'https://api.douban.com/v2/group/topic/{}/'  # id
COMMENTS_URL = BASE_URL + 'comments'
PER_PAGE_COUNT = 100

WEBSITE_URL = 'https://www.douban.com/group/'

HEADERS = {
    'Host': 'api.douban.com',
    'Referer': 'api.douban.com',
    'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) \
AppleWebKit/537.36 (KHTML, like Gecko) Chrome/38.0.2125.101 Safari/537.36'
}


def refine(text):
    text = text.replace('\r\n', '\n')
    return text


class Doubangroup(base.BaseNovel):

    def __init__(self, topic_id):
        super().__init__(utils.base_to_url(BASE_URL, topic_id),
                         tid=topic_id, cache=True)
        self.comments_url = utils.base_to_url(COMMENTS_URL, topic_id)

        self.req = self.session = None
        self.use_exist_session = False
        self.author_id = self.content = ''
        self.num_comments = 0

    def use_session(self, s):
        if s:
            self.session = s
            self.use_exist_session = True

    def run(self, refresh=True):
        if self.running and not refresh:
            return
        self.refine = refine
        self.req = requests.get(
            self.url, headers=self.headers, proxies=self.proxies
        ).json()
        self.title = self.req['title']
        self.author_id = self.req['author']['id']
        self.num_comments = self.req['comments_count']
        print(self.title)
        if self.cache:
            if not self.use_exist_session:
                self.session = db.new_session()
            self._add_website()
            self._add_article()
            self.session.flush()
        else:
            self.content = self.get_content()
        self.running = True

    def close(self):
        if self.cache and not self.use_exist_session:
            self.session.close()
        self.running = False

    # noinspection PyArgumentList
    def _add_website(self):
        website = self.session.query(models.Website).filter_by(
            name=self.source
        ).first()

        if not website:
            website = models.Website(name=self.source, url=WEBSITE_URL)
            self.session.add(website)

    # noinspection PyArgumentList
    def _add_article(self):
        article = self.session.query(models.Article).filter_by(
            id=self.tid, source=self.source
        ).first()

        if not article:
            article = models.Article(id=self.tid, title=self.title,
                                     text=self.get_content(), source=self.source)
            self.session.add(article)

    def get_content(self):
        content_list = [self.refine(self.req['content'])]

        for i in range(self.num_comments // PER_PAGE_COUNT + 1):
            params = {
                'start': i * PER_PAGE_COUNT,
                'count': PER_PAGE_COUNT
            }
            req = requests.get(
                self.comments_url, headers=self.headers,
                proxies=self.proxies, params=params
            ).json()
            for c in req['comments']:
                if c['author']['id'] != self.author_id:
                    continue
                content_list.append(self.refine(c['text']))

        content = '\n\n\n\n'.join(content_list)
        return content

    def dump(self):
        filename = utils.get_filename(self.title, overwrite=self.overwrite)
        filename = filename.replace('/', '_')
        print(filename)
        if self.cache:
            content = self.session.query(models.Article).filter_by(
                id=self.tid, source=self.source
            ).one().text
        else:
            content = self.content
        with open(filename, 'w') as fp:
            fp.write(content)

    def dump_and_close(self):
        self.run()
        self.dump()
        self.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from .. import serial, utils

BASE_URL = 'http://2bgif.com/chapters/{}'


class Bgif2(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         None, '#description',
                         chap_sel='tbody td',
                         chap_type=serial.ChapterType.path,
                         tid=tid)

    def get_title_and_author(self):
        st = self.doc('title').text()
        pat = re.compile(r'(\w+)\s+-\s+(\w+)\s+', re.U)
        return re.match(pat, st).groups()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pyquery import PyQuery

from .. import serial, utils, config

# The domain has been changed into `www.lwxs.la`,
# but it can be redirected correctly, so there is no need to change.
BASE_URL = 'http://www.lwxsw.org/books/{}/{}/'
INTRO_URL = 'http://www.lwxsw.org/book/{}/index.html'


class Lwxsw(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         utils.base_to_url(INTRO_URL, tid), '.intro',
                         chap_sel='.bookinfo_td td',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        name = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'og:novel:book_name'
        ).attr('content')

        author = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'og:novel:author'
        ).attr('content')

        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
from urllib.parse import urljoin

from pyquery import PyQuery

from .. import serial, utils

BASE_URL = 'http://www.dzxsw.la/book/{}/index.html'
INTRO_URL = 'http://www.dzxsw.la/book/{}/'


class Dzxsw(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         utils.base_to_url(INTRO_URL, tid), '#cintro',
                         tid=tid)

    def get_title_and_author(self):
        name = self.doc('.title').text()
        pat = re.compile(r'作者：(.+)', re.U)
        st = self.doc('.item').filter(
            lambda i, e: re.match(pat, PyQuery(e).text())
        )
        author = re.match(pat, st.text()).group(1)
        return name, author

    @property
    def chapter_list(self):
        clist = self.doc('.chapterList')('ul').eq(1)('li').filter(
            lambda i, e: PyQuery(e)('a').attr('href')
        ).map(
            lambda i, e: (
                i,
                urljoin(utils.get_base_url(self.url),
                        PyQuery(e)('a').attr('href')),
                PyQuery(e).text()
            )
        )
        return clist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils

BASE_URL = 'http://www.feizw.com/Html/{}/Index.html'
INTRO_URL = 'http://www.feizw.com/Book/{}/Index.aspx'


class FeizwTool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.extend(
            (re.compile(pat, re.I) for pat in
             (r'www.feizw.com 飞速中文网',
              r'最快更新无错小说阅读，请访问www.feizw.com',
              r'手机请访问：http://m.feizw.com'))
        )


class Feizw(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         utils.base_to_url(INTRO_URL, tid), '.intro',
                         chap_sel='.chapterlist li',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.tool = FeizwTool

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords').attr('content')
        name = re.match(r'(.*?),.*', st).group(1)

        st = self.doc('span:not([class])').text()
        author = re.match(r'文 / (\S*)', st).group(1)

        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .. import single, utils, config

BASE_URL = 'http://www.sis001.com/forum/archiver/tid-{}.html'


class Sis(single.SingleNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid),
                         '.archiver_postbody',
                         title_sel='h2',
                         title_type=single.TitleType.selector,
                         tid=tid)
        self.encoding = config.GB



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.zhaishu8.com/xiaoshuo/{}/{}/'
INTRO_URL = 'http://www.zhaishu8.com/book/{}/index.aspx'


class Zhaishu8Tool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.extend((
            re.compile(r'<h2>.*?</h2>'),
            re.compile(r'完结穿越小说推荐：')
        ))


class Zhaishu8(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#texts',
                         utils.base_to_url(INTRO_URL, tid), '#b_info',
                         chap_sel='#BookText li',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB
        self.tool = Zhaishu8Tool

    def get_title_and_author(self):
        st = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('name') == 'keywords').attr('content')
        return re.match(r'(.*?),(.*?),', st).groups()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from enum import Enum

from pyquery import PyQuery
from termcolor import colored

from .base import SinglePage
from .db import new_session
from .models import Article, Website
from .utils import get_filename, get_base_url


class TitleType(Enum):
    selector = 1
    meta = 2


class SingleNovel(SinglePage):

    def __init__(self, url, selector,
                 title_sel=None, title_type=None,
                 tid=None, cache=True):
        super().__init__(url, selector,
                         tid=tid, cache=cache)
        self.title_sel = title_sel
        self.title_type = title_type

        self.session = None
        self.use_exist_session = False

    def use_session(self, s):
        if s:
            self.session = s
            self.use_exist_session = True

    def run(self, refresh=False):
        super().run(refresh=refresh)
        print(colored(self.title, 'green'))
        if self.cache:
            if not self.use_exist_session:
                self.session = new_session()
            self._add_website()
            self._add_article()
            self.session.flush()

    def close(self):
        if self.cache and not self.use_exist_session:
            self.session.close()
        self.running = False

    # noinspection PyArgumentList
    def _add_website(self):
        website = self.session.query(Website).filter_by(
            name=self.source
        ).first()

        if not website:
            website = Website(name=self.source,
                              url=get_base_url(self.url))
            self.session.add(website)

    # noinspection PyArgumentList
    def _add_article(self):
        if self.tid is not None:
            article = self.session.query(Article).filter_by(
                id=self.tid, source=self.source
            ).first()
        else:
            article = None

        if not article:
            article = Article(id=self.tid, title=self.title,
                              text=self.get_content(), source=self.source)
            self.session.add(article)

    def get_title(self):
        if not self.title_sel:
            raise NotImplementedError('get_title')
        if self.title_type == TitleType.selector:
            return self.refine(self.doc(self.title_sel).html())
        elif self.title_type == TitleType.meta:
            return self.doc('meta').filter(
                lambda i, e:
                PyQuery(e).attr(self.title_sel[0]) == self.title_sel[1]
            ).attr('content')
        else:
            raise NameError('title_type')

    def get_content(self):
        if not self.selector:
            raise NotImplementedError('get_content')
        content = '\n\n\n\n'.join(
            self.doc(self.selector).map(
                lambda i, e: self.refine(PyQuery(e).html())
            )
        )
        return content

    def dump(self):
        filename = get_filename(self.title, overwrite=self.overwrite)
        print(filename)
        if self.cache:
            content = self.session.query(Article).filter_by(
                id=self.tid, source=self.source
            ).one().text
        else:
            content = self.content
        with open(filename, 'w') as fp:
            fp.write(self.title)
            fp.write('\n\n\n\n')
            fp.write(content)
            fp.write('\n')

    def dump_and_close(self):
        self.run()
        if self.cache:
            self.update_novel_list()
        self.dump()
        self.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from .. import serial, utils

BASE_URL = 'http://www.icnsp.com/plugin.php?id=ysysyan_novel:novel&do=chapter&tid={}'


class Icnsp(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#msgdiv',
                         None, '.noveldes',
                         chap_sel='.c_chapter td',
                         chap_type=serial.ChapterType.path,
                         tid=tid)

    def get_title_and_author(self):
        title = self.doc('.h2_subject').text()
        author = self.doc('.author')('a').text()
        return title, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import re
import sys
import textwrap

from novel import __version__
from novel import cli, config


class MyParser(argparse.ArgumentParser):

    def __init__(self):
        description = textwrap.dedent("""\
            Fetch novels from Internet, and write into file.

            Available sources:
              bgif2, biquge, dzxsw, feizw, haxtxt, klxsw, lwxs, lwxs520, lwxsw,
              piaotian, piaotiancc, ranwen, shu69, shushu8, sto, ttshuba,
              ttzw, ttzw5, uks5, wdxs, xs365, yfzww, yq33, zhaishu8,
              doubangroup, ...
        """)
        super().__init__(
            prog='fetchnovels',
            description=description,
            formatter_class=argparse.RawTextHelpFormatter
        )
        self.add_argument('-V', '--version', action='version',
                          version=__version__)

        group = self.add_mutually_exclusive_group()
        group.add_argument('-u', '--update', action='store_true',
                           help='update novels in the database')
        group.add_argument('-d', '--dump-only', action='store_true',
                           help='dump only without update')
        group.add_argument('-l', '--list', action='store_true',
                           help='list novels in the database')
        group.add_argument('-ls', '--list-serial', action='store_true',
                           help='list serials in the database')
        group.add_argument('-la', '--list-article', action='store_true',
                           help='list articles in the database')
        group.add_argument('-D', '--delete', action='store_true',
                           help='delete novels in the database')
        group.add_argument('-m', '--mark-finish', action='store_true',
                           help='mark novels as finished')

        self.add_argument('-v', '--verbose', action='count',
                          help='show in more detail')
        self.add_argument('-r', '--refresh', action='store_true',
                          help='refresh novel in the database')

        proxy_group = self.add_mutually_exclusive_group()
        proxy_group.add_argument('-p', '--proxy', action='store',
                                 help='use specific proxy')
        proxy_group.add_argument('-n', '--no-proxy', action='store_true',
                                 help='do not use any proxies')

        self.add_argument('source', nargs='?',
                          help='download source')
        self.add_argument('tid', nargs='*',
                          help='id for novels to download')


def main():
    parser = MyParser()
    args = parser.parse_args()

    if args.source:
        need_fix = re.match(r'(\d+)(.+)', args.source)
        if need_fix:
            source = '{g[1]}{g[0]}'.format(g=need_fix.groups())
        else:
            source = args.source
    else:
        source = None

    if args.no_proxy:
        proxies = '---'
    else:
        proxies = args.proxy

    config.check_first()

    with cli.NovelCmdline(source, args.tid, proxies, args.verbose) as fac:
        if args.list:
            fac.list()
        elif args.list_serial:
            fac.list_serial()
        elif args.list_article:
            fac.list_article()
        elif args.delete:
            fac.delete()
        elif args.mark_finish:
            fac.try_mark_finish()
        elif args.update:
            if args.refresh:
                fac.refresh()
            else:
                fac.update()
        elif args.dump_only:
            fac.dump()
        elif source:
            print('{}: {}'.format(source, args.tid))
            if not args.tid:
                print('No specific tid to download!')
                sys.exit(1)
            if args.refresh:
                fac.refresh()
            else:
                fac.update()
            fac.dump()
        else:
            parser.print_usage()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from .. import serial, utils, config

BASE_URL = 'http://www.shushu8.com/{}/'


class Shushu8(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#content',
                         None, '.bookintro',
                         chap_sel='li',
                         chap_type=serial.ChapterType.last,
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        st = self.doc('title').text()
        pat = re.compile(r'(.+)全文阅读_(.+)作品.*')
        return re.match(pat, st).groups()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from urllib.parse import urljoin

from pyquery import PyQuery

from .. import serial, utils, config

BASE_URL = 'http://www.klxsw.com/files/article/html/{}/{}/'


class Klxsw(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '#r1c',
                         tid=tid)
        self.encoding = config.GB

    def get_title_and_author(self):
        name = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:book_name'
        ).attr('content')

        author = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:author'
        ).attr('content')

        return name, author

    def get_intro(self):
        intro = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:description'
        ).attr('content')
        return intro

    @property
    def chapter_list(self):
        clist = self.doc('div').filter(
            lambda i, e: PyQuery(e).attr('align') == 'left'
        ).filter(
            lambda i, e: PyQuery(e)('a').attr('href')
        ).map(
            lambda i, e: (i,
                          urljoin(self.url,
                                  PyQuery(e)('a').attr('href').strip()),
                          PyQuery(e).text())
        )
        return clist



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

import pypandoc
from setuptools import setup, find_packages

from novel import __version__


if __name__ == '__main__':
    assert sys.version_info.major > 2, 'Only works with python3'
    setup(
        name='FetchNovels',
        version=__version__,
        description='Fetch novels from Internet',
        long_description=pypandoc.convert_file('README.md', 'rst'),
        license='GPLv3',
        url='https://github.com/wangjiezhe/FetchNovels',

        author='wangjiezhe',
        author_email='wangjiezhe@gmail.com',

        packages=find_packages(exclude=['docs', 'tools', 'tests*']),
        entry_points={
            'console_scripts': ['fetchnovels=novel.main:main'],
        },

        classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Natural Language :: English',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
        ]
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pyquery import PyQuery

from .. import serial, utils

BASE_URL = 'http://www.5uks.com/book/{}/'


class Uks5Page(serial.Page):

    def get_content(self):
        content = self.doc(self.selector).eq(1).html()
        content = self.refine(content)
        return content


class Uks5(serial.SerialNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid), '.box_box',
                         chap_sel='.list_box li',
                         chap_type=serial.ChapterType.path,
                         tid=tid)
        self.page = Uks5Page

    def get_title_and_author(self):
        name = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:book_name'
        ).attr('content')

        author = self.doc('meta').filter(
            lambda i, e: PyQuery(e).attr('property') == 'og:novel:author'
        ).attr('content')

        return name, author



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from .. import single, utils, config

BASE_URL = 'http://www.cool18.com/bbs4/index.php?app=forum&act=threadview&tid={}'


class Cool18Tool(utils.Tool):

    def __init__(self):
        super().__init__()
        self.remove_extras.append(
            re.compile(r'<.*?bodyend.*?>.*')
        )



class Cool18(single.SingleNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid),
                         'pre',
                         title_sel=('name', 'Description'),
                         title_type=single.TitleType.meta,
                         tid=tid)
        self.tool = Cool18Tool
        self.encoding = config.GB



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Some constants
"""

import json
import os
from random import randrange

GOAGENT = {'http': '127.0.0.1:8087'}
GB = 'GB18030'
BIG = 'Big5'

UAS = (
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) \
Chrome/51.0.2704.84 Safari/537.36',
    'Mozilla/5.0 (X11; Linux x86_64; rv:47.0) Gecko/20100101 Firefox/47.0',
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) \
Chrome/50.0.2661.76 YaBrowser/16.6.0.6383 (beta) Safari/537.36',
)

HOME_DIR = os.path.expanduser('~')
CACHE_DIR = os.path.join(HOME_DIR, '.cache', 'novel')
CACHE_DB = os.path.join(CACHE_DIR, 'cache.db')
NOVEL_LIST_JSON = os.path.join(CACHE_DIR, 'novel_list.json')


def get_headers():
    ua = UAS[randrange(len(UAS))]
    headers = {'User-Agent': ua}
    return headers


def check_first():
    if not os.path.exists(CACHE_DIR):
        os.makedirs(CACHE_DIR)


def load_novel_list():
    if not os.path.exists(NOVEL_LIST_JSON):
        return dict()

    with open(NOVEL_LIST_JSON) as fp:
        nl = json.load(fp)
    return nl


def update_novel_list(nl, source, tid):
    if source in nl:
        if tid not in nl[source]:
            nl[source].append(tid)
    else:
        nl[source] = [tid]
    return nl


def save_novel_list(nl):
    with open(NOVEL_LIST_JSON, 'w') as fp:
        json.dump(nl, fp, indent=2)


def update_and_save_novel_list(source, tid):
    nl = load_novel_list()
    nl = update_novel_list(nl, source, tid)
    save_novel_list(nl)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from enum import Enum
from multiprocessing.dummy import Pool
from urllib.parse import urljoin

from pyquery import PyQuery
from termcolor import colored

from .base import BaseNovel, SinglePage
from .db import new_session
from .models import Serial, Chapter, Website
from .utils import get_base_url, get_filename


class Page(SinglePage):

    def __init__(self, url, title, selector,
                 headers=None, proxies=None,
                 encoding='UTF-8', tool=None):
        super().__init__(url, selector, headers, proxies, encoding, tool)
        self.title = title

    def dump(self, path=None, folder=None, num=None):
        if not path:
            if num is not None:
                pre = '「{:d}」'.format(num)
            else:
                pre = ''
            filename = '{}{}'.format(
                pre, get_filename(self.title, overwrite=self.overwrite))
            if not folder:
                path = os.path.join(os.getcwd(), filename)
            else:
                path = os.path.join(folder, filename)
        print(self.title)
        with open(path, 'w') as fp:
            fp.write(self.title)
            fp.write('\n\n\n')
            fp.write(self.content)
            fp.write('\n')

    def dump_and_close(self, path=None, folder=None, num=None):
        self.run()
        self.dump(path, folder, num)


class IntroPage(SinglePage):

    def __init__(self, url, selector,
                 headers=None, proxies=None, encoding='UTF-8',
                 tool=None):
        super().__init__(url, selector, headers, proxies, encoding, tool)
        self.title = 'Introduction'


class ChapterType(Enum):
    whole = 1
    path = 2
    last = 3


class SerialNovel(BaseNovel):

    def __init__(self, url, cont_sel,
                 intro_url=None, intro_sel=None,
                 chap_sel=None, chap_type=None,
                 tid=None, cache=True):
        super().__init__(url, tid=tid, cache=cache)
        self.cont_sel = cont_sel
        self.intro_url = intro_url
        self.intro_sel = intro_sel
        self.page = Page
        self.intro_page = IntroPage
        self.chap_sel = chap_sel
        self.chap_type = chap_type

        self.finish = False
        self.session = None
        self.use_exist_session = False

    def use_session(self, s):
        if s:
            self.session = s
            self.use_exist_session = True

    def run(self, refresh=False):
        super().run(refresh=refresh)
        self.title, self.author = self.get_title_and_author()
        print(colored(self.title, 'green'), self.author)

        if self.cache:
            if not self.use_exist_session:
                self.session = new_session()
            self._add_website()
            self._add_novel()
            self._update_chapters()
            self.session.flush()

    # noinspection PyArgumentList
    def _add_website(self):
        website = self.session.query(Website).filter_by(
            name=self.source
        ).first()

        if not website:
            website = Website(name=self.source,
                              url=get_base_url(self.url))
            self.session.add(website)

    # noinspection PyArgumentList
    def _add_novel(self):
        if self.tid is not None:
            novel = self.session.query(Serial).filter_by(
                id=self.tid, source=self.source
            ).first()
        else:
            novel = None

        if not novel:
            novel = Serial(id=self.tid, title=self.title, author=self.author,
                           intro=self.get_intro(), source=self.source)
            self.session.add(novel)

            novel.chapters = [Chapter(id=tid, title=title, url=url)
                              for tid, url, title in self.chapter_list]
        else:
            old_chapters_ids = self.session.query(Chapter.id).filter_by(
                novel_id=self.tid, novel_source=self.source
            ).all()
            old_chapters_ids = list(*zip(*old_chapters_ids))
            novel.chapters.extend(
                [Chapter(id=cid, title=title, url=url)
                 for cid, url, title in self.chapter_list
                 if cid not in old_chapters_ids])

        if self.finish:
            novel.finish = True

    def _update_chapters(self):
        empty_chapters = self.session.query(Chapter).filter_by(
            novel_id=self.tid, novel_source=self.source
        ).filter(Chapter.text.is_(None))

        with Pool(100) as p:
            p.map(self._update_chapter, empty_chapters, 10)

    def _update_chapter(self, ch):
        print(ch.title)
        page = self.page(
            ch.url, ch.title, self.cont_sel,
            None, self.proxies, self.encoding,
            self.tool
        )
        page.run()
        ch.text = page.content

    def close(self):
        if self.cache and not self.use_exist_session:
            self.session.close()
        self.running = False

    def get_title_and_author(self):
        return NotImplementedError('get_title_and_author')

    @property
    def chapter_list(self):
        if self.chap_sel and self.chap_type:
            return self._chapter_list_with_sel(self.chap_sel, self.chap_type)
        raise NotImplementedError('chapter_list')

    def _chapter_list_with_sel(self, selector, chap_type):
        clist = self.doc(selector).filter(
            lambda i, e: PyQuery(e)('a').attr('href')
        )
        if chap_type == ChapterType.whole:
            clist = clist.map(
                lambda i, e: (i,
                              PyQuery(e)('a').attr('href'),
                              PyQuery(e).text())
            )
        elif chap_type == ChapterType.path:
            clist = clist.map(
                lambda i, e: (i,
                              urljoin(get_base_url(self.url),
                                      PyQuery(e)('a').attr('href')),
                              PyQuery(e).text())
            )
        elif chap_type == ChapterType.last:
            clist = clist.map(
                lambda i, e: (i,
                              urljoin(self.url, PyQuery(e)('a').attr('href')),
                              PyQuery(e).text())
            )
        else:
            raise NameError('chap_type')
        return clist

    def get_intro(self):
        if not self.intro_url:
            if not self.intro_sel:
                return ''
            intro = self.doc(self.intro_sel).html()
            intro = self.refine(intro)
            return intro
        else:
            intro_page = self.intro_page(
                self.intro_url, self.intro_sel,
                None, self.proxies, self.encoding,
                self.tool)
            intro_page.run()
            return intro_page.content

    @property
    def download_dir(self):
        return os.path.join(os.getcwd(),
                            '《{self.title}》{self.author}'.format(self=self))

    def _dump_split(self):
        download_dir = self.download_dir
        if not os.path.isdir(download_dir):
            os.makedirs(download_dir)
        print('《{self.title}》{self.author}'.format(self=self))

        if self.cache:
            intro = self.session.query(Serial).filter_by(
                id=self.tid, source=self.source
            ).one().intro
        else:
            intro = self.get_intro()
        path = os.path.join(download_dir, 'Introduction.txt')
        with open(path, 'w') as fp:
            fp.write('Introduction')
            fp.write('\n\n\n\n')
            fp.write(intro)
            fp.write('\n')

        if self.cache:
            novel = self.session.query(Serial).filter_by(
                id=self.tid, source=self.source
            ).one()
            for ch in novel.chapters:
                filename = '「{:d}」{}.txt'.format(ch.id + 1, ch.title)
                path = os.path.join(download_dir, filename)
                with open(path, 'w') as fp:
                    fp.write(ch.title)
                    fp.write('\n\n\n\n')
                    fp.write(ch.text)
                    fp.write('\n')
        else:
            for i, url, title in self.chapter_list:
                self._dump_chapter(url, title, i + 1)

    def _dump_chapter(self, url, title, num):
        page = self.page(
            url, title, self.cont_sel,
            None, self.proxies, self.encoding,
            self.tool)
        page.dump_and_close(folder=self.download_dir, num=num)

    def dump_split(self):
        self.run()
        self._dump_split()
        self.close()

    def dump(self):
        filename = get_filename(self.title, self.author, self.overwrite)
        print(filename)

        with open(filename, 'w') as fp:
            fp.write(self.title)
            fp.write('\n\n')
            fp.write(self.author)

            fp.write('\n\n\n')
            if self.cache:
                novel = self.session.query(Serial).filter_by(
                    id=self.tid, source=self.source
                ).one()
                fp.write(novel.intro)
                for ch in novel.chapters:
                    fp.write('\n\n\n\n')
                    fp.write(ch.title)
                    fp.write('\n\n\n')
                    fp.write(ch.text)
                    fp.write('\n')
            else:
                fp.write(self.get_intro())
                for _, url, title in self.chapter_list:
                    fp.write('\n\n\n\n')
                    fp.write(title)
                    fp.write('\n\n\n')
                    fp.write(self._get_chapter(url, title))
                    fp.write('\n')

    def _get_chapter(self, url, title):
        print(title)
        page = self.page(
            url, title, self.cont_sel,
            None, self.proxies, self.encoding,
            self.tool
        )
        page.run()
        return page.content

    def dump_and_close(self):
        self.run()
        if self.cache:
            self.update_novel_list()
        self.dump()
        self.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re

from .. import single, utils, config

BASE_URL = 'http://bbs.6park.com/bbs4/messages/{}.html'


class Park6(single.SingleNovel):

    def __init__(self, tid):
        super().__init__(utils.base_to_url(BASE_URL, tid),
                         None,
                         title_sel='title',
                         title_type=single.TitleType.selector,
                         tid=tid)
        self.encoding = config.GB

    def get_content(self):
        content = self.doc.html()
        pat = re.compile(r'.*<!--bodybegin-->(.*)<!--bodyend-->.*',
                         re.S)
        content = re.match(pat, content).group(1)
        content = self.refine(content)
        return content



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from urllib.error import HTTPError
from urllib.parse import urlparse

import pypinyin
from lxml.etree import XMLSyntaxError
from pyquery import PyQuery
from requests import ConnectionError

from .config import get_headers, update_and_save_novel_list
from .decorators import retry
from .utils import Tool


class BaseNovel(object):

    def __init__(self, url,
                 headers=None, proxies=None,
                 encoding='UTF-8', tool=None,
                 tid=None, cache=False):
        self.url = url
        self._headers = headers or get_headers()
        self._proxies = proxies
        self.encoding = encoding
        self.tool = tool or Tool
        self._tid = tid
        self.cache = cache

        self.running = False
        self.overwrite = True
        self.refine = self.doc = None
        self.title = self.author = ''

    @property
    def tid(self):
        if self._tid is not None:
            return str(self._tid)
        else:
            tp = pypinyin.slug(self.title, errors='ignore', separator='_')
            ap = pypinyin.slug(self.author, errors='ignore', separator='_')
            tid = '{} {}'.format(tp, ap)
        return tid

    @classmethod
    def get_source_from_class(cls):
        return cls.__name__.lower()

    def get_source_from_url(self):
        source = urlparse(self.url).netloc
        source = source.lstrip('www.').replace('.', '_')
        return source

    @property
    def source(self):
        return self.get_source_from_class()

    def run(self, refresh=False):
        if self.running and not refresh:
            return
        self.refine = self.tool().refine
        self.doc = self.get_doc()
        self.running = True

    def close(self):
        return

    def update_novel_list(self):
        update_and_save_novel_list(self.source, self.tid)

    @retry((HTTPError, XMLSyntaxError, ConnectionError))
    def get_doc(self):
        return PyQuery(url=self.url, headers=self.headers,
                       proxies=self.proxies, encoding=self.encoding)

    @property
    def headers(self):
        return self._headers

    @headers.setter
    def headers(self, value):
        self._headers = value or {}

    @property
    def proxies(self):
        return self._proxies

    @proxies.setter
    def proxies(self, value):
        self._proxies = value or {}

    def dump(self):
        raise NotImplementedError('dump')

    def dump_and_close(self):
        self.run()
        self.dump()
        self.close()


class SinglePage(BaseNovel):

    def __init__(self, url, selector,
                 headers=None, proxies=None,
                 encoding='UTF-8', tool=None,
                 tid=None, cache=False):
        super().__init__(url, headers, proxies, encoding, tool, tid, cache)
        self.selector = selector

        self.content = ''

    def run(self, refresh=False):
        super().run(refresh=refresh)
        if not self.title:
            self.title = self.get_title()
        if not self.cache:
            self.content = self.get_content()

    def get_content(self):
        if not self.selector:
            return ''
        content = self.doc(self.selector).html()
        content = self.refine(content)
        return content

    def get_title(self):
        if self.title:
            return self.title
        else:
            raise NotImplementedError('get_title')

    def dump(self):
        filename = '{self.title}.txt'.format(self=self)
        print(self.title)
        with open(filename, 'w') as fp:
            fp.write(self.title)
            fp.write('\n\n\n\n')
            fp.write(self.content)
            fp.write('\n')
